"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.fetchWeda = exports.fetchCloud = void 0;
const cloud_api_1 = require("@cloudbase/cloud-api");
const utils_1 = require("./utils");
const config_1 = require("./config");
const error_1 = require("./error");
const logger_1 = require("./logger");
const customLogger = logger_1.logger.log.bind(logger_1.logger);
/** 请求云API的默认参数 */
const DEFAULT_CLOUD_OPTIONS = {
    service: 'lowcode',
    version: '2021-01-08',
    // 30s
    timeout: 30000,
    data: {},
    capitalizeDataKey: true,
};
/**
 * 请求云API
 * @param params 配置参数
 */
async function fetchCloud(params) {
    try {
        const { TENCENTCLOUD_SECRETID, TENCENTCLOUD_SECRETKEY, TENCENTCLOUD_SESSIONTOKEN } = config_1.getCloudbaseContext();
        const newOptions = Object.assign({}, DEFAULT_CLOUD_OPTIONS, params);
        const service = cloud_api_1.CloudApiService.getInstance({
            service: newOptions.service,
            version: newOptions.version,
            credential: {
                secretId: TENCENTCLOUD_SECRETID,
                secretKey: TENCENTCLOUD_SECRETKEY,
                token: TENCENTCLOUD_SESSIONTOKEN,
            },
            timeout: newOptions.timeout,
        });
        const resp = await service.request(params.action, newOptions.capitalizeDataKey ? utils_1.capitalizeKey(newOptions.data) : newOptions.data);
        return resp;
    }
    catch (error) {
        normalizeCloudAPIError(error, { type: 'design', action: params.action });
    }
}
exports.fetchCloud = fetchCloud;
/** 默认参数 */
const DEFAULT_WEDA_OPTIONS = {
    data: {},
    capitalizeDataKey: true,
    requestId: true,
};
/**
 * 请求Weda后台接口
 * @param params 配置参数
 */
async function fetchWeda(params) {
    try {
        const userSource = config_1.getUserSource();
        if (!userSource || !userSource.uid || !userSource.source) {
            throw new error_1.TCBError('UserError.InvalidUser', error_1.COMMON_ERROR_MESSAGE['UserError.InvalidUser'], { reason: `can get user source, unable to call weda-runtime action ${params.action}` });
        }
        const { serviceType } = params;
        const newParams = Object.assign({}, DEFAULT_WEDA_OPTIONS, params, {
            uid: userSource.uid,
            source: userSource.source,
        });
        delete newParams.serviceType;
        const requestId = newParams.requestId === true
            ? config_1.getConfig('requestId')
            : newParams.requestId || undefined;
        const envInfo = config_1.getCloudbaseContext();
        const requestParams = Object.assign(Object.assign({}, newParams), { 
            // @ts-ignore
            envType: config_1.getConfig('envType'), requestId, envId: envInfo.SCF_NAMESPACE, credentials: {
                secretId: envInfo.TENCENTCLOUD_SECRETID,
                secretKey: envInfo.TENCENTCLOUD_SECRETKEY,
                sessionToken: envInfo.TENCENTCLOUD_SESSIONTOKEN,
            }, clientIp: envInfo.TCB_SOURCE_IP || envInfo.WX_CLIENTIP, data: newParams.capitalizeDataKey
                ? utils_1.capitalizeKey(newParams.data)
                : newParams.data });
        const resp = await cloud_api_1.wedaRequest(requestParams, {
            // @ts-ignore
            serviceType,
            target: config_1.getConfig('wedaTarget'),
            logger: customLogger,
            proxy: process.env.WEDA_PROXY,
        });
        return resp;
    }
    catch (error) {
        normalizeCloudAPIError(error, { type: 'runtime', action: params.action });
    }
}
exports.fetchWeda = fetchWeda;
/** 标准花错误信息 */
function normalizeCloudAPIError(error, options) {
    const reason = `failed to weda-${options.type} action ${options.action} on env ${config_1.getConfig('wedaTarget') || 'prod'}`;
    if (error.name === 'CloudBaseError') {
        const extraMsg = error.requestId ? ` [requestId]${error.requestId}` : '';
        throw new error_1.TCBError(`InnerError.${error.code || 'Unknown'}`, error.original.Message + extraMsg, {
            original: error,
            reason: `${reason}, ${error.original ? JSON.stringify(error.original) : ''} ${extraMsg}`,
        });
    }
    if (error.name === 'TCBError')
        throw error;
    throw new error_1.TCBError('InnerError', `请求微搭后台失败, ${error.message}`, {
        original: error,
        reason: `${reason}, ${error.name} ${error.message}`,
    });
}
