import tcb from '@cloudbase/node-sdk';
import { IPlatformDataSource } from '@cloudbase/cals';
import { UriOptions, CoreOptions, RequestResponse } from 'request';
export declare type IRequestConfig = UriOptions & CoreOptions;
export interface IAuthData {
    /** 后台token接口返回的原始认证信息 */
    authInfo: Record<string, any>;
    /** 请求地址 uri */
    uri?: string;
    /** 自定义头 */
    headers?: Record<string, any>;
    /** body */
    body?: Record<string, any>;
    /** 查询字符串 */
    qs?: Record<string, any>;
}
/**
 * 第三方数据源的认证对象
 */
export interface IHttpAuthProvider {
    getAuthData: (dsId: string, requestConfig: IRequestConfig) => Promise<IAuthData>;
    transformResponse?: (body: any, response: RequestResponse) => Promise<Record<string, any>>;
}
/**
 * 云函数 context 中用的第三方数据源认证对象, 由 IHttpAuthProvider bind 数据源id 而来
 */
export interface IHttpAuthProvider4Context {
    getAuthData: (requestConfig: IRequestConfig) => Promise<IAuthData>;
    /** 转换结果 */
    transformResponse?: IHttpAuthProvider['transformResponse'];
}
/** 用户来源信息 */
export interface IUserSource {
    /** 用户ID */
    uid: string;
    /** uid 类型, 1.tcb 2.微信 3.企业微信 4.weda内部 */
    source: 1 | 2 | 3 | 4;
}
/** 环境变量信息 */
export interface IEnvInfo {
    /** 微搭应用ID, 仅从微搭应用前端调用才有该值 */
    wedaAppId?: string;
    openId?: string;
    /** 微信中, 多个小程序共享环境时, 来源用户的openId, 实际使用时应当优先使用该值 */
    fromOpenId?: string;
    /** 用户在来源小程序中的openid */
    currentOpenId?: string;
    appId?: string;
    /** 微信中, 多个小程序共享环境时, 来源小程序的appId, 实际使用时应当优先使用该值 */
    fromAppId?: string;
    /** 来源小程序的appid */
    currentAppId?: string;
    uid?: string;
    customUserId?: string;
    isAnonymous: boolean;
    /** 数据源名称, 用户在数据源管理中创建的数据源才有该属性 */
    dataSourceName?: string;
    /** 数据源完整名称, 带数据源ID, 为部署后底层(云函数/数据库)实际使用的标志 */
    dataSourceFullName?: string;
    /** 是否为预览环境, 即当前云函数接受到的请求是否来自实时预览或体验应用发起的 */
    isPreview: boolean;
    /**
     * 环境类型
     *  pre: 预览环境, 即请求均由实时预览或体验应用发起
     *  prod: 正式环境, 即请求均有发布后的应用发起
     */
    envType: 'prod' | 'pre';
    /** 云开发环境ID */
    envId: string;
    /**
     * tcb 调用的来源信息, 可用于判断是否被其他云函数调用, 或者被云API调用等
     */
    tcbSource: string;
}
/**
 * 调用数据源的参数
 */
export interface ICallDataSourceParams {
    /** 数据源名称(即数据源标志) */
    dataSourceName: string;
    /** 数据源方法名称 */
    methodName: string;
    /** 数据源方法参数 */
    params?: any;
}
/** 数据源函数执行的 context */
export interface IFnContext {
    /**
     * 调用数据源方法, 可用于调用其他数据源的方法
     */
    callDataSource: (options: ICallDataSourceParams) => Promise<unknown>;
    /**
     * 云开发node sdk 对象
     *  即 `import tcb from '@cloudbase/node-sdk'` 引入的 `tcb`
     *  使用文档 https://docs.cloudbase.net/api-reference/server/node-sdk/introduction.html
     */
    cloudbase: typeof tcb;
    /**
     * 云开发 node sdk 初始化后返回的 app 对象
     *  即 `const app = tcb.init({...})` 得到的 app. 初始化默认使用当前低码使用的云开发环境相关参数进行初始化
     *  可使用 app 直接调用 云开发云函数及存储相关能力(如app.callFunction({...})、app.uploadFile({...}))
     *  使用文档 云函数 https://docs.cloudbase.net/api-reference/server/node-sdk/functions.html
     *        存储 https://docs.cloudbase.net/api-reference/server/node-sdk/storage.html
     */
    app: tcb.CloudBase;
    /**
     * 云开发 node sdk 的鉴权对象
     *  即 上述 app.auth() 返回得到的 auth 对象
     *  可用于直接调用鉴权相关的能力如(如 auth.getUserInfo(), auth.getEndUserInfo() 等等)
     *  使用文档 https://docs.cloudbase.net/api-reference/server/node-sdk/auth.html
     */
    auth: ReturnType<tcb.CloudBase['auth']>;
    /**
     * 云开发 node sdk 的数据库对象
     *  即 app.database() 返回的对象
     *  可使用 database 获取集合的引用(database.collection('<collection-name>')), 访问指令对象(database.command)等等
     *  使用文档 https://docs.cloudbase.net/api-reference/server/node-sdk/database/database.html
     */
    database: tcb.Database.Db;
    /**
     * 当前数据源关联的数据库表的引用对象(云开发 node sdk 的集合引用对象), 仅自建数据源才有该属性
     *  即 `context.database.collection(context.env.dataSourceFullName)` 返回的对象
     *  可直接使用该对象当前数据源的数据库表进行读写操作
     *  使用文档 https://docs.cloudbase.net/api-reference/server/node-sdk/database/database.html
     */
    collection?: tcb.Database.CollectionReference;
    /**
     * 根据数据源标识获取数据源在云开发环境中的数据库表名称
     */
    getCollectionName: (dsName: string) => string;
    /** 环境信息 */
    env: IEnvInfo;
    /**
     * 第三方数据源的auth处理对象, 仅使用非空白模版创建的第三方数据源有该对象
     */
    httpAuth?: IHttpAuthProvider4Context;
    /**
     * 环境变量信息
     * @deprecated 使用 env 替代
     */
    envInfo: IEnvInfo;
    /** 当前数据源的公共变量 */
    vars: Record<string, string>;
    /** 通过微信 cloudID 获取的信息 */
    weixin: Record<string, any> | null;
    /** 数据源描述信息 */
    dataSourceProfile?: IPlatformDataSource;
    /** 数据源请求携带的默认参数 */
    defaultParams: any;
    /** 请求的微搭后台接入层地址, 可通过设置为 prod/pre 来指定地址 */
    wedaTarget?: string;
}
/** 数据源函数的参数 */
export declare type IFnParams = Record<string | number, any>;
/** 数据源方法类型 */
export declare type IDsFn = {
    /** http 类型方法需有该属性 */
    isHttp?: boolean;
    (params: IFnParams, context: IFnContext): Promise<any>;
};
/** 数据源基础信息 */
export interface IDsKeyInfo {
    /** 数据源ID */
    id: string;
    /** 数据源名称 */
    name: string;
    /** 数据源类型 database, cloud-integration */
    type: string;
}
/**
 * 通过callFunction方式调用本云函数入参
 *  注意: query, variables, operationName 这三个名称被 graphql 的参数占用, 在 ICloudFnParams 不要使用
 */
export interface ICloudFnParams {
    /**
     * 路由模块名称
     *  使用 ROUTE_NAMES 的 key, 或者直接使用 router 的值 均可, 默认值为 ROUTE_NAMES.datasource (数据源)
     */
    mode?: string;
    /**
     * 数据源名称
     *  仅当 mode 为 ROUTE_NAMES.datasource (数据源) 有意义
     */
    dataSourceName?: string;
    /**
     * 方法名称
     *  仅当 mode 为 ROUTE_NAMES.datasource(数据源) 或 ROUTE_NAMES.common(公共模块) 有意义
     */
    methodName?: string;
    /**
     * 是否为测试模式, 仅当当前云函数为预览环境时, 方可进入测试模式
     */
    isTestMode?: boolean;
    /**
     * 被测试的数据源详情, 仅当 isTestMode 为 true 时, 该字段有意义
     */
    testDataSource?: IPlatformDataSource;
    /**
     * 方法的默认参数
     *  仅当调用数据源方法时有意义
     */
    defaultParams?: any;
    /**
     * 数据库数据源的viewId, 记录数据源版本信息
     *  仅调用数据源方法时有意义
     *  已废弃, 后端将使用最新的数据源版本
     * @deprecated
     */
    viewId?: string;
    /**
     * 方法参数
     *  各个模块均需要参数, 参数根据调用的方法而异, 部分方法不需要参数
     */
    params?: any;
    /**
     * 调用的引用环境类型
     *  pre: 预览环境
     *  prod: 正式环境
     *
     * 在云开发的环境变量中若无法通过 IS_PREVIEW 当前环境是否为预览, 则使用该参数
     */
    envType?: 'pre' | 'prod';
    /**
     * 请求的微搭后台接入层地址, 可通过设置为 prod/pre 来指定地址
     * 可以是下边三个值
     *  'prod' 正式地址
     *  'pre' 预发地址
     *  url 任意一个url
     */
    wedaTarget?: string;
    /**
     * 用户来源信息
     *  优先会从环境变量信息中解析
     *  仅当环境信息中取不到, 且调用来自其他云函数时, 方从该字段获取用户来源信息
     */
    userSource?: IUserSource;
    /**
     * 前端应用的 ua 信息, 仅做统计分析之用
     *  若使用 http 方式调用, 则可从 http headers 中获取该信息
     */
    userAgent?: string;
    /**
     * 前端应用发起数据源请求的页面地址,  仅做统计分析之用
     *  若使用 http 方式调用, 则可从 http headers 中获取该信息
     */
    referrer?: string;
    /**
     * weda gateway 代理
     *  正常仅在本地开发模式需要设置该值
     */
    proxy?: string;
    /** 微搭应用ID */
    wedaAppId?: string;
    /**
     * 其他额外信息
     *  包括 wx_ 打头的微信授权拿到的 cloudID
     */
    [k: string]: any;
}
